@@ -1,29 +1,26 @@
 """Reads vehicle status from MyBMW portal."""
+
 from __future__ import annotations
 
 import logging
-from typing import Any
 
-from bimmer_connected.vehicle import MyBMWVehicle
 import voluptuous as vol
 
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_DEVICE_ID, CONF_ENTITY_ID, CONF_NAME, Platform
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers import discovery, entity_registry as er
-import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.device_registry import DeviceInfo
-from homeassistant.helpers.typing import ConfigType
-from homeassistant.helpers.update_coordinator import CoordinatorEntity
+from homeassistant.helpers import (
+    config_validation as cv,
+    device_registry as dr,
+    discovery,
+    entity_registry as er,
+)
 
-from .const import ATTR_VIN, ATTRIBUTION, CONF_READ_ONLY, DATA_HASS_CONFIG, DOMAIN
-from .coordinator import BMWDataUpdateCoordinator
+from .const import ATTR_VIN, CONF_READ_ONLY, DOMAIN
+from .coordinator import BMWConfigEntry, BMWDataUpdateCoordinator
 
 _LOGGER = logging.getLogger(__name__)
 
 
-CONFIG_SCHEMA = cv.removed(DOMAIN, raise_if_present=False)
-
 SERVICE_SCHEMA = vol.Schema(
     vol.Any(
         {vol.Required(ATTR_VIN): cv.string},
@@ -50,31 +47,25 @@ PLATFORMS = [
 SERVICE_UPDATE_STATE = "update_state"
 
 
-async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
-    """Set up the BMW Connected Drive component from configuration.yaml."""
-    # Store full yaml config in data for platform.NOTIFY
-    hass.data.setdefault(DOMAIN, {})
-    hass.data[DOMAIN][DATA_HASS_CONFIG] = config
-
-    return True
-
-
 @callback
 def _async_migrate_options_from_data_if_missing(
-    hass: HomeAssistant, entry: ConfigEntry
+    hass: HomeAssistant, entry: BMWConfigEntry
 ) -> None:
     data = dict(entry.data)
     options = dict(entry.options)
 
     if CONF_READ_ONLY in data or list(options) != list(DEFAULT_OPTIONS):
-        options = dict(DEFAULT_OPTIONS, **options)
+        options = dict(
+            DEFAULT_OPTIONS,
+            **{k: v for k, v in options.items() if k in DEFAULT_OPTIONS},
+        )
         options[CONF_READ_ONLY] = data.pop(CONF_READ_ONLY, False)
 
         hass.config_entries.async_update_entry(entry, data=data, options=options)
 
 
 async def _async_migrate_entries(
-    hass: HomeAssistant, config_entry: ConfigEntry
+    hass: HomeAssistant, config_entry: BMWConfigEntry
 ) -> bool:
     """Migrate old entry."""
     entity_registry = er.async_get(hass)
@@ -82,11 +73,29 @@ async def _async_migrate_entries(
     @callback
     def update_unique_id(entry: er.RegistryEntry) -> dict[str, str] | None:
         replacements = {
-            "charging_level_hv": "remaining_battery_percent",
-            "fuel_percent": "remaining_fuel_percent",
+            Platform.SENSOR.value: {
+                "charging_level_hv": "fuel_and_battery.remaining_battery_percent",
+                "fuel_percent": "fuel_and_battery.remaining_fuel_percent",
+                "ac_current_limit": "charging_profile.ac_current_limit",
+                "charging_start_time": "fuel_and_battery.charging_start_time",
+                "charging_end_time": "fuel_and_battery.charging_end_time",
+                "charging_status": "fuel_and_battery.charging_status",
+                "charging_target": "fuel_and_battery.charging_target",
+                "remaining_battery_percent": "fuel_and_battery.remaining_battery_percent",
+                "remaining_range_total": "fuel_and_battery.remaining_range_total",
+                "remaining_range_electric": "fuel_and_battery.remaining_range_electric",
+                "remaining_range_fuel": "fuel_and_battery.remaining_range_fuel",
+                "remaining_fuel": "fuel_and_battery.remaining_fuel",
+                "remaining_fuel_percent": "fuel_and_battery.remaining_fuel_percent",
+                "activity": "climate.activity",
+            }
         }
-        if (key := entry.unique_id.split("-")[-1]) in replacements:
-            new_unique_id = entry.unique_id.replace(key, replacements[key])
+        if (key := entry.unique_id.split("-")[-1]) in replacements.get(
+            entry.domain, []
+        ):
+            new_unique_id = entry.unique_id.replace(
+                key, replacements[entry.domain][key]
+            )
             _LOGGER.debug(
                 "Migrating entity '%s' unique_id from '%s' to '%s'",
                 entry.entity_id,
@@ -112,7 +121,7 @@ async def _async_migrate_entries(
     return True
 
 
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
+async def async_setup_entry(hass: HomeAssistant, entry: BMWConfigEntry) -> bool:
     """Set up BMW Connected Drive from a config entry."""
 
     _async_migrate_options_from_data_if_missing(hass, entry)
@@ -122,12 +131,11 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     # Set up one data coordinator per account/config entry
     coordinator = BMWDataUpdateCoordinator(
         hass,
-        entry=entry,
+        config_entry=entry,
     )
     await coordinator.async_config_entry_first_refresh()
 
-    hass.data.setdefault(DOMAIN, {})
-    hass.data[DOMAIN][entry.entry_id] = coordinator
+    entry.runtime_data = coordinator
 
     # Set up all platforms except notify
     await hass.config_entries.async_forward_entry_setups(
@@ -142,54 +150,28 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
             Platform.NOTIFY,
             DOMAIN,
             {CONF_NAME: DOMAIN, CONF_ENTITY_ID: entry.entry_id},
-            hass.data[DOMAIN][DATA_HASS_CONFIG],
+            {},
         )
     )
 
+    # Clean up vehicles which are not assigned to the account anymore
+    account_vehicles = {(DOMAIN, v.vin) for v in coordinator.account.vehicles}
+    device_registry = dr.async_get(hass)
+    device_entries = dr.async_entries_for_config_entry(
+        device_registry, config_entry_id=entry.entry_id
+    )
+    for device in device_entries:
+        if not device.identifiers.intersection(account_vehicles):
+            device_registry.async_update_device(
+                device.id, remove_config_entry_id=entry.entry_id
+            )
+
     return True
 
 
-async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
+async def async_unload_entry(hass: HomeAssistant, entry: BMWConfigEntry) -> bool:
     """Unload a config entry."""
-    unload_ok = await hass.config_entries.async_unload_platforms(
+
+    return await hass.config_entries.async_unload_platforms(
         entry, [platform for platform in PLATFORMS if platform != Platform.NOTIFY]
     )
-
-    if unload_ok:
-        hass.data[DOMAIN].pop(entry.entry_id)
-
-    return unload_ok
-
-
-class BMWBaseEntity(CoordinatorEntity[BMWDataUpdateCoordinator]):
-    """Common base for BMW entities."""
-
-    coordinator: BMWDataUpdateCoordinator
-    _attr_attribution = ATTRIBUTION
-    _attr_has_entity_name = True
-
-    def __init__(
-        self,
-        coordinator: BMWDataUpdateCoordinator,
-        vehicle: MyBMWVehicle,
-    ) -> None:
-        """Initialize entity."""
-        super().__init__(coordinator)
-
-        self.vehicle = vehicle
-
-        self._attrs: dict[str, Any] = {
-            "car": self.vehicle.name,
-            "vin": self.vehicle.vin,
-        }
-        self._attr_device_info = DeviceInfo(
-            identifiers={(DOMAIN, self.vehicle.vin)},
-            manufacturer=vehicle.brand.name,
-            model=vehicle.name,
-            name=vehicle.name,
-        )
-
-    async def async_added_to_hass(self) -> None:
-        """When entity is added to hass."""
-        await super().async_added_to_hass()
-        self._handle_coordinator_update()
