@@ -1,9 +1,14 @@
 """Helper functions for the Cert Expiry platform."""
+
+import asyncio
+import datetime
 import socket
 import ssl
+from typing import Any
 
 from homeassistant.core import HomeAssistant
-from homeassistant.util import dt
+from homeassistant.util import dt as dt_util
+from homeassistant.util.ssl import get_default_context
 
 from .const import TIMEOUT
 from .errors import (
@@ -14,30 +19,38 @@ from .errors import (
 )
 
 
-def get_cert(
+async def async_get_cert(
+    hass: HomeAssistant,
     host: str,
     port: int,
-):
+) -> dict[str, Any]:
     """Get the certificate for the host and port combination."""
-    ctx = ssl.create_default_context()
-    address = (host, port)
-    with socket.create_connection(address, timeout=TIMEOUT) as sock:
-        with ctx.wrap_socket(sock, server_hostname=address[0]) as ssock:
-            cert = ssock.getpeercert()
-            return cert
+    async with asyncio.timeout(TIMEOUT):
+        transport, _ = await hass.loop.create_connection(
+            asyncio.Protocol,
+            host,
+            port,
+            ssl=get_default_context(),
+            happy_eyeballs_delay=0.25,
+            server_hostname=host,
+        )
+    try:
+        return transport.get_extra_info("peercert")  # type: ignore[no-any-return]
+    finally:
+        transport.close()
 
 
 async def get_cert_expiry_timestamp(
     hass: HomeAssistant,
     hostname: str,
     port: int,
-):
+) -> datetime.datetime:
     """Return the certificate's expiration timestamp."""
     try:
-        cert = await hass.async_add_executor_job(get_cert, hostname, port)
+        cert = await async_get_cert(hass, hostname, port)
     except socket.gaierror as err:
         raise ResolveFailed(f"Cannot resolve hostname: {hostname}") from err
-    except socket.timeout as err:
+    except TimeoutError as err:
         raise ConnectionTimeout(
             f"Connection timeout with server: {hostname}:{port}"
         ) from err
@@ -51,4 +64,4 @@ async def get_cert_expiry_timestamp(
         raise ValidationFailure(err.args[0]) from err
 
     ts_seconds = ssl.cert_time_to_seconds(cert["notAfter"])
-    return dt.utc_from_timestamp(ts_seconds)
+    return dt_util.utc_from_timestamp(ts_seconds)
