@@ -1,11 +1,14 @@
 """Runtime entry data for ESPHome stored in hass.data."""
+
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Callable, Coroutine, Iterable
+from collections import defaultdict
+from collections.abc import Callable, Iterable
 from dataclasses import dataclass, field
 from functools import partial
 import logging
+from operator import delitem
 from typing import TYPE_CHECKING, Any, Final, TypedDict, cast
 
 from aioesphomeapi import (
@@ -18,13 +21,18 @@ from aioesphomeapi import (
     CameraState,
     ClimateInfo,
     CoverInfo,
+    DateInfo,
+    DateTimeInfo,
     DeviceInfo,
     EntityInfo,
     EntityState,
+    Event,
+    EventInfo,
     FanInfo,
     LightInfo,
     LockInfo,
     MediaPlayerInfo,
+    MediaPlayerSupportedFormat,
     NumberInfo,
     SelectInfo,
     SensorInfo,
@@ -32,22 +40,28 @@ from aioesphomeapi import (
     SwitchInfo,
     TextInfo,
     TextSensorInfo,
+    TimeInfo,
+    UpdateInfo,
     UserService,
+    ValveInfo,
     build_unique_id,
 )
 from aioesphomeapi.model import ButtonInfo
 from bleak_esphome.backend.device import ESPHomeBluetoothDevice
 
+from homeassistant.components.assist_satellite import AssistSatelliteConfiguration
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import Platform
 from homeassistant.core import CALLBACK_TYPE, HomeAssistant, callback
 from homeassistant.helpers import entity_registry as er
-from homeassistant.helpers.dispatcher import async_dispatcher_send
 from homeassistant.helpers.storage import Store
 
 from .const import DOMAIN
 from .dashboard import async_get_dashboard
 
+type ESPHomeConfigEntry = ConfigEntry[RuntimeEntryData]
+
+
 INFO_TO_COMPONENT_TYPE: Final = {v: k for k, v in COMPONENT_TYPE_TO_INFO.items()}
 
 _SENTINEL = object()
@@ -62,6 +76,9 @@ INFO_TYPE_TO_PLATFORM: dict[type[EntityInfo], Platform] = {
     CameraInfo: Platform.CAMERA,
     ClimateInfo: Platform.CLIMATE,
     CoverInfo: Platform.COVER,
+    DateInfo: Platform.DATE,
+    DateTimeInfo: Platform.DATETIME,
+    EventInfo: Platform.EVENT,
     FanInfo: Platform.FAN,
     LightInfo: Platform.LIGHT,
     LockInfo: Platform.LOCK,
@@ -72,9 +89,28 @@ INFO_TYPE_TO_PLATFORM: dict[type[EntityInfo], Platform] = {
     SwitchInfo: Platform.SWITCH,
     TextInfo: Platform.TEXT,
     TextSensorInfo: Platform.SENSOR,
+    TimeInfo: Platform.TIME,
+    UpdateInfo: Platform.UPDATE,
+    ValveInfo: Platform.VALVE,
 }
 
 
+def build_device_unique_id(mac: str, entity_info: EntityInfo) -> str:
+    """Build unique ID for entity, appending @device_id if it belongs to a sub-device.
+
+    This wrapper around build_unique_id ensures that entities belonging to sub-devices
+    have their device_id appended to the unique_id to handle proper migration when
+    entities move between devices.
+    """
+    base_unique_id = build_unique_id(mac, entity_info)
+
+    # If entity belongs to a sub-device, append @device_id
+    if entity_info.device_id:
+        return f"{base_unique_id}@{entity_info.device_id}"
+
+    return base_unique_id
+
+
 class StoreData(TypedDict, total=False):
     """ESPHome storage data."""
 
@@ -95,7 +131,9 @@ class RuntimeEntryData:
     title: str
     client: APIClient
     store: ESPHomeStorage
-    state: dict[type[EntityState], dict[int, EntityState]] = field(default_factory=dict)
+    state: defaultdict[type[EntityState], dict[int, EntityState]] = field(
+        default_factory=lambda: defaultdict(dict)
+    )
     # When the disconnect callback is called, we mark all states
     # as stale so we will always dispatch a state update when the
     # device reconnects. This is the same format as state_subscriptions.
@@ -107,29 +145,38 @@ class RuntimeEntryData:
     device_info: DeviceInfo | None = None
     bluetooth_device: ESPHomeBluetoothDevice | None = None
     api_version: APIVersion = field(default_factory=APIVersion)
-    cleanup_callbacks: list[Callable[[], None]] = field(default_factory=list)
-    disconnect_callbacks: set[Callable[[], None]] = field(default_factory=set)
-    state_subscriptions: dict[
-        tuple[type[EntityState], int], Callable[[], None]
-    ] = field(default_factory=dict)
+    cleanup_callbacks: list[CALLBACK_TYPE] = field(default_factory=list)
+    disconnect_callbacks: set[CALLBACK_TYPE] = field(default_factory=set)
+    state_subscriptions: dict[tuple[type[EntityState], int], CALLBACK_TYPE] = field(
+        default_factory=dict
+    )
+    device_update_subscriptions: set[CALLBACK_TYPE] = field(default_factory=set)
+    static_info_update_subscriptions: set[Callable[[list[EntityInfo]], None]] = field(
+        default_factory=set
+    )
     loaded_platforms: set[Platform] = field(default_factory=set)
     platform_load_lock: asyncio.Lock = field(default_factory=asyncio.Lock)
     _storage_contents: StoreData | None = None
     _pending_storage: Callable[[], StoreData] | None = None
-    assist_pipeline_update_callbacks: list[Callable[[], None]] = field(
-        default_factory=list
-    )
+    assist_pipeline_update_callbacks: list[CALLBACK_TYPE] = field(default_factory=list)
     assist_pipeline_state: bool = False
     entity_info_callbacks: dict[
         type[EntityInfo], list[Callable[[list[EntityInfo]], None]]
     ] = field(default_factory=dict)
-    entity_info_key_remove_callbacks: dict[
-        tuple[type[EntityInfo], int], list[Callable[[], Coroutine[Any, Any, None]]]
-    ] = field(default_factory=dict)
     entity_info_key_updated_callbacks: dict[
         tuple[type[EntityInfo], int], list[Callable[[EntityInfo], None]]
     ] = field(default_factory=dict)
     original_options: dict[str, Any] = field(default_factory=dict)
+    media_player_formats: dict[str, list[MediaPlayerSupportedFormat]] = field(
+        default_factory=lambda: defaultdict(list)
+    )
+    assist_satellite_config_update_callbacks: list[
+        Callable[[AssistSatelliteConfiguration], None]
+    ] = field(default_factory=list)
+    assist_satellite_set_wake_word_callbacks: list[Callable[[str], None]] = field(
+        default_factory=list
+    )
+    device_id_to_name: dict[int, str] = field(default_factory=dict)
 
     @property
     def name(self) -> str:
@@ -145,16 +192,6 @@ class RuntimeEntryData:
             "_", " "
         )
 
-    @property
-    def signal_device_updated(self) -> str:
-        """Return the signal to listen to for core device state update."""
-        return f"esphome_{self.entry_id}_on_device_update"
-
-    @property
-    def signal_static_info_updated(self) -> str:
-        """Return the signal to listen to for updates on static info."""
-        return f"esphome_{self.entry_id}_on_list"
-
     @callback
     def async_register_static_info_callback(
         self,
@@ -164,39 +201,7 @@ class RuntimeEntryData:
         """Register to receive callbacks when static info changes for an EntityInfo type."""
         callbacks = self.entity_info_callbacks.setdefault(entity_info_type, [])
         callbacks.append(callback_)
-        return partial(
-            self._async_unsubscribe_register_static_info, callbacks, callback_
-        )
-
-    @callback
-    def _async_unsubscribe_register_static_info(
-        self,
-        callbacks: list[Callable[[list[EntityInfo]], None]],
-        callback_: Callable[[list[EntityInfo]], None],
-    ) -> None:
-        """Unsubscribe to when static info is registered."""
-        callbacks.remove(callback_)
-
-    @callback
-    def async_register_key_static_info_remove_callback(
-        self,
-        static_info: EntityInfo,
-        callback_: Callable[[], Coroutine[Any, Any, None]],
-    ) -> CALLBACK_TYPE:
-        """Register to receive callbacks when static info is removed for a specific key."""
-        callback_key = (type(static_info), static_info.key)
-        callbacks = self.entity_info_key_remove_callbacks.setdefault(callback_key, [])
-        callbacks.append(callback_)
-        return partial(self._async_unsubscribe_static_key_remove, callbacks, callback_)
-
-    @callback
-    def _async_unsubscribe_static_key_remove(
-        self,
-        callbacks: list[Callable[[], Coroutine[Any, Any, None]]],
-        callback_: Callable[[], Coroutine[Any, Any, None]],
-    ) -> None:
-        """Unsubscribe to when static info is removed."""
-        callbacks.remove(callback_)
+        return partial(callbacks.remove, callback_)
 
     @callback
     def async_register_key_static_info_updated_callback(
@@ -208,18 +213,7 @@ class RuntimeEntryData:
         callback_key = (type(static_info), static_info.key)
         callbacks = self.entity_info_key_updated_callbacks.setdefault(callback_key, [])
         callbacks.append(callback_)
-        return partial(
-            self._async_unsubscribe_static_key_info_updated, callbacks, callback_
-        )
-
-    @callback
-    def _async_unsubscribe_static_key_info_updated(
-        self,
-        callbacks: list[Callable[[EntityInfo], None]],
-        callback_: Callable[[EntityInfo], None],
-    ) -> None:
-        """Unsubscribe to when static info is updated ."""
-        callbacks.remove(callback_)
+        return partial(callbacks.remove, callback_)
 
     @callback
     def async_set_assist_pipeline_state(self, state: bool) -> None:
@@ -230,20 +224,14 @@ class RuntimeEntryData:
 
     @callback
     def async_subscribe_assist_pipeline_update(
-        self, update_callback: Callable[[], None]
-    ) -> Callable[[], None]:
+        self, update_callback: CALLBACK_TYPE
+    ) -> CALLBACK_TYPE:
         """Subscribe to assist pipeline updates."""
         self.assist_pipeline_update_callbacks.append(update_callback)
-        return partial(self._async_unsubscribe_assist_pipeline_update, update_callback)
+        return partial(self.assist_pipeline_update_callbacks.remove, update_callback)
 
     @callback
-    def _async_unsubscribe_assist_pipeline_update(
-        self, update_callback: Callable[[], None]
-    ) -> None:
-        """Unsubscribe to assist pipeline updates."""
-        self.assist_pipeline_update_callbacks.remove(update_callback)
-
-    async def async_remove_entities(
+    def async_remove_entities(
         self, hass: HomeAssistant, static_infos: Iterable[EntityInfo], mac: str
     ) -> None:
         """Schedule the removal of an entity."""
@@ -251,18 +239,12 @@ class RuntimeEntryData:
         ent_reg = er.async_get(hass)
         for info in static_infos:
             if entry := ent_reg.async_get_entity_id(
-                INFO_TYPE_TO_PLATFORM[type(info)], DOMAIN, build_unique_id(mac, info)
+                INFO_TYPE_TO_PLATFORM[type(info)],
+                DOMAIN,
+                build_device_unique_id(mac, info),
             ):
                 ent_reg.async_remove(entry)
 
-        callbacks: list[Coroutine[Any, Any, None]] = []
-        for static_info in static_infos:
-            callback_key = (type(static_info), static_info.key)
-            if key_callbacks := self.entity_info_key_remove_callbacks.get(callback_key):
-                callbacks.extend([callback_() for callback_ in key_callbacks])
-        if callbacks:
-            await asyncio.gather(*callbacks)
-
     @callback
     def async_update_entity_infos(self, static_infos: Iterable[EntityInfo]) -> None:
         """Call static info updated callbacks."""
@@ -272,7 +254,10 @@ class RuntimeEntryData:
                 callback_(static_info)
 
     async def _ensure_platforms_loaded(
-        self, hass: HomeAssistant, entry: ConfigEntry, platforms: set[Platform]
+        self,
+        hass: HomeAssistant,
+        entry: ESPHomeConfigEntry,
+        platforms: set[Platform],
     ) -> None:
         async with self.platform_load_lock:
             if needed := platforms - self.loaded_platforms:
@@ -280,17 +265,26 @@ class RuntimeEntryData:
             self.loaded_platforms |= needed
 
     async def async_update_static_infos(
-        self, hass: HomeAssistant, entry: ConfigEntry, infos: list[EntityInfo], mac: str
+        self,
+        hass: HomeAssistant,
+        entry: ESPHomeConfigEntry,
+        infos: list[EntityInfo],
+        mac: str,
     ) -> None:
         """Distribute an update of static infos to all platforms."""
         # First, load all platforms
-        needed_platforms = set()
-        if async_get_dashboard(hass):
-            needed_platforms.add(Platform.UPDATE)
+        needed_platforms: set[Platform] = set()
 
-        if self.device_info and self.device_info.voice_assistant_version:
-            needed_platforms.add(Platform.BINARY_SENSOR)
-            needed_platforms.add(Platform.SELECT)
+        if self.device_info:
+            if async_get_dashboard(hass):
+                # Only load the update platform if the device_info is set
+                # When we restore the entry, the device_info may not be set yet
+                # and we don't want to load the update platform since it needs
+                # a complete device_info.
+                needed_platforms.add(Platform.UPDATE)
+            if self.device_info.voice_assistant_feature_flags_compat(self.api_version):
+                needed_platforms.add(Platform.BINARY_SENSOR)
+                needed_platforms.add(Platform.SELECT)
 
         ent_reg = er.async_get(hass)
         registry_get_entity = ent_reg.async_get_entity_id
@@ -303,7 +297,8 @@ class RuntimeEntryData:
             if (
                 (old_unique_id := info.unique_id)
                 and (old_entry := registry_get_entity(platform, DOMAIN, old_unique_id))
-                and (new_unique_id := build_unique_id(mac, info)) != old_unique_id
+                and (new_unique_id := build_device_unique_id(mac, info))
+                != old_unique_id
                 and not registry_get_entity(platform, DOMAIN, new_unique_id)
             ):
                 ent_reg.async_update_entity(old_entry, new_unique_id=new_unique_id)
@@ -312,40 +307,49 @@ class RuntimeEntryData:
 
         # Make a dict of the EntityInfo by type and send
         # them to the listeners for each specific EntityInfo type
-        infos_by_type: dict[type[EntityInfo], list[EntityInfo]] = {}
+        infos_by_type: defaultdict[type[EntityInfo], list[EntityInfo]] = defaultdict(
+            list
+        )
         for info in infos:
-            info_type = type(info)
-            if info_type not in infos_by_type:
-                infos_by_type[info_type] = []
-            infos_by_type[info_type].append(info)
+            infos_by_type[type(info)].append(info)
 
-        callbacks_by_type = self.entity_info_callbacks
-        for type_, entity_infos in infos_by_type.items():
-            if callbacks_ := callbacks_by_type.get(type_):
-                for callback_ in callbacks_:
-                    callback_(entity_infos)
+        for type_, callbacks in self.entity_info_callbacks.items():
+            # If all entities for a type are removed, we
+            # still need to call the callbacks with an empty list
+            # to make sure the entities are removed.
+            entity_infos = infos_by_type.get(type_, [])
+            for callback_ in callbacks:
+                callback_(entity_infos)
 
-        # Then send dispatcher event
-        async_dispatcher_send(hass, self.signal_static_info_updated, infos)
+        # Finally update static info subscriptions
+        for callback_ in self.static_info_update_subscriptions:
+            callback_(infos)
+
+    @callback
+    def async_subscribe_device_updated(self, callback_: CALLBACK_TYPE) -> CALLBACK_TYPE:
+        """Subscribe to state updates."""
+        self.device_update_subscriptions.add(callback_)
+        return partial(self.device_update_subscriptions.remove, callback_)
+
+    @callback
+    def async_subscribe_static_info_updated(
+        self, callback_: Callable[[list[EntityInfo]], None]
+    ) -> CALLBACK_TYPE:
+        """Subscribe to static info updates."""
+        self.static_info_update_subscriptions.add(callback_)
+        return partial(self.static_info_update_subscriptions.remove, callback_)
 
     @callback
     def async_subscribe_state_update(
         self,
         state_type: type[EntityState],
         state_key: int,
-        entity_callback: Callable[[], None],
-    ) -> Callable[[], None]:
+        entity_callback: CALLBACK_TYPE,
+    ) -> CALLBACK_TYPE:
         """Subscribe to state updates."""
         subscription_key = (state_type, state_key)
         self.state_subscriptions[subscription_key] = entity_callback
-        return partial(self._async_unsubscribe_state_update, subscription_key)
-
-    @callback
-    def _async_unsubscribe_state_update(
-        self, subscription_key: tuple[type[EntityState], int]
-    ) -> None:
-        """Unsubscribe to state updates."""
-        self.state_subscriptions.pop(subscription_key)
+        return partial(delitem, self.state_subscriptions, subscription_key)
 
     @callback
     def async_update_state(self, state: EntityState) -> None:
@@ -359,9 +363,9 @@ class RuntimeEntryData:
         if (
             current_state == state
             and subscription_key not in stale_state
-            and state_type is not CameraState
+            and state_type not in (CameraState, Event)
             and not (
-                state_type is SensorState  # noqa: E721
+                state_type is SensorState
                 and (platform_info := self.info.get(SensorInfo))
                 and (entity_info := platform_info.get(state.key))
                 and (cast(SensorInfo, entity_info)).force_update
@@ -373,16 +377,17 @@ class RuntimeEntryData:
         if subscription := self.state_subscriptions.get(subscription_key):
             try:
                 subscription()
-            except Exception as ex:  # pylint: disable=broad-except
+            except Exception:
                 # If we allow this exception to raise it will
                 # make it all the way to data_received in aioesphomeapi
                 # which will cause the connection to be closed.
-                _LOGGER.exception("Error while calling subscription: %s", ex)
+                _LOGGER.exception("Error while calling subscription")
 
     @callback
-    def async_update_device_state(self, hass: HomeAssistant) -> None:
+    def async_update_device_state(self) -> None:
         """Distribute an update of a core device state like availability."""
-        async_dispatcher_send(hass, self.signal_device_updated)
+        for callback_ in self.device_update_subscriptions.copy():
+            callback_()
 
     async def async_load_from_store(self) -> tuple[list[EntityInfo], list[UserService]]:
         """Load the retained data from store and return de-serialized data."""
@@ -406,7 +411,7 @@ class RuntimeEntryData:
         ]
         return infos, services
 
-    async def async_save_to_store(self) -> None:
+    def async_save_to_store(self) -> None:
         """Generate dynamic data to store and save it to the filesystem."""
         if TYPE_CHECKING:
             assert self.device_info is not None
@@ -441,7 +446,7 @@ class RuntimeEntryData:
             await self.store.async_save(self._pending_storage())
 
     async def async_update_listener(
-        self, hass: HomeAssistant, entry: ConfigEntry
+        self, hass: HomeAssistant, entry: ESPHomeConfigEntry
     ) -> None:
         """Handle options update."""
         if self.original_options == entry.options:
@@ -483,3 +488,35 @@ class RuntimeEntryData:
         # We use this to determine if a deep sleep device should
         # be marked as unavailable or not.
         self.expected_disconnect = True
+
+    @callback
+    def async_register_assist_satellite_config_updated_callback(
+        self,
+        callback_: Callable[[AssistSatelliteConfiguration], None],
+    ) -> CALLBACK_TYPE:
+        """Register to receive callbacks when the Assist satellite's configuration is updated."""
+        self.assist_satellite_config_update_callbacks.append(callback_)
+        return partial(self.assist_satellite_config_update_callbacks.remove, callback_)
+
+    @callback
+    def async_assist_satellite_config_updated(
+        self, config: AssistSatelliteConfiguration
+    ) -> None:
+        """Notify listeners that the Assist satellite configuration has been updated."""
+        for callback_ in self.assist_satellite_config_update_callbacks.copy():
+            callback_(config)
+
+    @callback
+    def async_register_assist_satellite_set_wake_word_callback(
+        self,
+        callback_: Callable[[str], None],
+    ) -> CALLBACK_TYPE:
+        """Register to receive callbacks when the Assist satellite's wake word is set."""
+        self.assist_satellite_set_wake_word_callbacks.append(callback_)
+        return partial(self.assist_satellite_set_wake_word_callbacks.remove, callback_)
+
+    @callback
+    def async_assist_satellite_set_wake_word(self, wake_word_id: str) -> None:
+        """Notify listeners that the Assist satellite wake word has been set."""
+        for callback_ in self.assist_satellite_set_wake_word_callbacks.copy():
+            callback_(wake_word_id)
