@@ -54,12 +54,15 @@ class HistoryStats:
         self._period = (MIN_TIME_UTC, MIN_TIME_UTC)
         self._state: HistoryStatsState = HistoryStatsState(None, None, self._period)
         self._history_current_period: list[HistoryState] = []
-        self._previous_run_before_start = False
+        self._has_recorder_data = False
         self._entity_states = set(entity_states)
         self._duration = duration
         self._start = start
         self._end = end
 
+        self._pending_events: list[Event[EventStateChangedData]] = []
+        self._query_count = 0
+
     async def async_update(
         self, event: Event[EventStateChangedData] | None
     ) -> HistoryStatsState:
@@ -85,23 +88,31 @@ class HistoryStats:
         utc_now = dt_util.utcnow()
         now_timestamp = floored_timestamp(utc_now)
 
+        # If we end up querying data from the recorder when we get triggered by a new state
+        # change event, it is possible this function could be reentered a second time before
+        # the first recorder query returns. In that case a second recorder query will be done
+        # and we need to hold the new event so that we can append it after the second query.
+        # Otherwise the event will be dropped.
+        if event:
+            self._pending_events.append(event)
+
         if current_period_start_timestamp > now_timestamp:
             # History cannot tell the future
             self._history_current_period = []
-            self._previous_run_before_start = True
+            self._has_recorder_data = False
             self._state = HistoryStatsState(None, None, self._period)
             return self._state
         #
         # We avoid querying the database if the below did NOT happen:
         #
-        # - The previous run happened before the start time
-        # - The start time changed
-        # - The period shrank in size
+        # - No previous run occurred (uninitialized)
+        # - The start time moved back in time
+        # - The end time moved back in time
         # - The previous period ended before now
         #
         if (
-            not self._previous_run_before_start
-            and current_period_start_timestamp == previous_period_start_timestamp
+            self._has_recorder_data
+            and current_period_start_timestamp >= previous_period_start_timestamp
             and (
                 current_period_end_timestamp == previous_period_end_timestamp
                 or (
@@ -110,36 +121,50 @@ class HistoryStats:
                 )
             )
         ):
+            start_changed = (
+                current_period_start_timestamp != previous_period_start_timestamp
+            )
+            end_changed = current_period_end_timestamp != previous_period_end_timestamp
+            if start_changed:
+                self._prune_history_cache(current_period_start_timestamp)
+
             new_data = False
             if event and (new_state := event.data["new_state"]) is not None:
-                if (
-                    current_period_start_timestamp
-                    <= floored_timestamp(new_state.last_changed)
-                    <= current_period_end_timestamp
+                if current_period_start_timestamp <= floored_timestamp(
+                    new_state.last_changed
                 ):
                     self._history_current_period.append(
                         HistoryState(new_state.state, new_state.last_changed_timestamp)
                     )
                     new_data = True
-            if not new_data and current_period_end_timestamp < now_timestamp:
+            if (
+                not new_data
+                and current_period_end_timestamp < now_timestamp
+                and not start_changed
+                and not end_changed
+            ):
                 # If period has not changed and current time after the period end...
                 # Don't compute anything as the value cannot have changed
                 return self._state
         else:
             await self._async_history_from_db(
-                current_period_start_timestamp, current_period_end_timestamp
+                current_period_start_timestamp, now_timestamp
             )
-            if event and (new_state := event.data["new_state"]) is not None:
-                if (
-                    current_period_start_timestamp
-                    <= floored_timestamp(new_state.last_changed)
-                    <= current_period_end_timestamp
-                ):
-                    self._history_current_period.append(
-                        HistoryState(new_state.state, new_state.last_changed_timestamp)
-                    )
+            for pending_event in self._pending_events:
+                if (new_state := pending_event.data["new_state"]) is not None:
+                    if current_period_start_timestamp <= floored_timestamp(
+                        new_state.last_changed
+                    ):
+                        self._history_current_period.append(
+                            HistoryState(
+                                new_state.state, new_state.last_changed_timestamp
+                            )
+                        )
 
-            self._previous_run_before_start = False
+            self._has_recorder_data = True
+
+        if self._query_count == 0:
+            self._pending_events.clear()
 
         seconds_matched, match_count = self._async_compute_seconds_and_changes(
             now_timestamp,
@@ -155,12 +180,16 @@ class HistoryStats:
         current_period_end_timestamp: float,
     ) -> None:
         """Update history data for the current period from the database."""
-        instance = get_instance(self.hass)
-        states = await instance.async_add_executor_job(
-            self._state_changes_during_period,
-            current_period_start_timestamp,
-            current_period_end_timestamp,
-        )
+        self._query_count += 1
+        try:
+            instance = get_instance(self.hass)
+            states = await instance.async_add_executor_job(
+                self._state_changes_during_period,
+                current_period_start_timestamp,
+                current_period_end_timestamp,
+            )
+        finally:
+            self._query_count -= 1
         self._history_current_period = [
             HistoryState(state.state, state.last_changed.timestamp())
             for state in states
@@ -198,6 +227,9 @@ class HistoryStats:
             current_state_matches = history_state.state in self._entity_states
             state_change_timestamp = history_state.last_changed
 
+            if math.floor(state_change_timestamp) > end_timestamp:
+                break
+
             if math.floor(state_change_timestamp) > now_timestamp:
                 # Shouldn't count states that are in the future
                 _LOGGER.debug(
@@ -205,7 +237,7 @@ class HistoryStats:
                     state_change_timestamp,
                     now_timestamp,
                 )
-                continue
+                break
 
             if previous_state_matches:
                 elapsed += state_change_timestamp - last_state_change_timestamp
@@ -223,3 +255,18 @@ class HistoryStats:
         # Save value in seconds
         seconds_matched = elapsed
         return seconds_matched, match_count
+
+    def _prune_history_cache(self, start_timestamp: float) -> None:
+        """Remove unnecessary old data from the history state cache from previous runs.
+
+        Update the timestamp of the last record from before the start to the current start time.
+        """
+        trim_count = 0
+        for i, history_state in enumerate(self._history_current_period):
+            if history_state.last_changed >= start_timestamp:
+                break
+            history_state.last_changed = start_timestamp
+            if i > 0:
+                trim_count += 1
+        if trim_count:  # Don't slice if no data was removed
+            self._history_current_period = self._history_current_period[trim_count:]
