@@ -1,4 +1,5 @@
 """Manage the Silicon Labs Multiprotocol add-on."""
+
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
@@ -10,18 +11,23 @@ from typing import Any, Protocol
 import voluptuous as vol
 import yarl
 
-from homeassistant import config_entries
 from homeassistant.components.hassio import (
     AddonError,
     AddonInfo,
     AddonManager,
     AddonState,
     hostname_from_addon_slug,
-    is_hassio,
+)
+from homeassistant.config_entries import (
+    ConfigEntry,
+    ConfigFlowResult,
+    OptionsFlow,
+    OptionsFlowManager,
 )
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.data_entry_flow import AbortFlow, FlowResult
+from homeassistant.data_entry_flow import AbortFlow
 from homeassistant.exceptions import HomeAssistantError
+from homeassistant.helpers.hassio import is_hassio
 from homeassistant.helpers.integration_platform import (
     async_process_integration_platforms,
 )
@@ -138,7 +144,10 @@ class MultiprotocolAddonManager(WaitingAddonManager):
     async def async_setup(self) -> None:
         """Set up the manager."""
         await async_process_integration_platforms(
-            self._hass, "silabs_multiprotocol", self._register_multipan_platform
+            self._hass,
+            "silabs_multiprotocol",
+            self._register_multipan_platform,
+            wait_for_platforms=True,
         )
         await self.async_load()
 
@@ -295,13 +304,12 @@ def is_multiprotocol_url(url: str) -> bool:
     return parsed.host == hostname
 
 
-class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
+class OptionsFlowHandler(OptionsFlow, ABC):
     """Handle an options flow for the Silicon Labs Multiprotocol add-on."""
 
-    def __init__(self, config_entry: config_entries.ConfigEntry) -> None:
+    def __init__(self, config_entry: ConfigEntry) -> None:
         """Set up the options flow."""
-        # pylint: disable-next=import-outside-toplevel
-        from homeassistant.components.zha.radio_manager import (
+        from homeassistant.components.zha.radio_manager import (  # noqa: PLC0415
             ZhaMultiPANMigrationHelper,
         )
 
@@ -309,7 +317,6 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
         self.start_task: asyncio.Task | None = None
         self.stop_task: asyncio.Task | None = None
         self._zha_migration_mgr: ZhaMultiPANMigrationHelper | None = None
-        self.config_entry = config_entry
         self.original_addon_config: dict[str, Any] | None = None
         self.revert_reason: str | None = None
 
@@ -334,7 +341,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
         """Return the ZHA name."""
 
     @property
-    def flow_manager(self) -> config_entries.OptionsFlowManager:
+    def flow_manager(self) -> OptionsFlowManager:
         """Return the correct flow manager."""
         return self.hass.config_entries.options
 
@@ -363,7 +370,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_init(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Manage the options."""
         if not is_hassio(self.hass):
             return self.async_abort(reason="not_hassio")
@@ -372,7 +379,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_on_supervisor(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle logic when on Supervisor host."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
         addon_info = await self._async_get_addon_info(multipan_manager)
@@ -383,7 +390,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_addon_not_installed(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle logic when the addon is not yet installed."""
         if user_input is None:
             return self.async_show_form(
@@ -400,7 +407,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_install_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Install Silicon Labs Multiprotocol add-on."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
 
@@ -408,6 +415,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
             self.install_task = self.hass.async_create_task(
                 multipan_manager.async_install_addon_waiting(),
                 "SiLabs Multiprotocol addon install",
+                eager_start=False,
             )
 
         if not self.install_task.done():
@@ -430,7 +438,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_install_failed(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Add-on installation failed."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
         return self.async_abort(
@@ -440,18 +448,13 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_configure_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Configure the Silicon Labs Multiprotocol add-on."""
-        # pylint: disable-next=import-outside-toplevel
-        from homeassistant.components.zha import DOMAIN as ZHA_DOMAIN
-
-        # pylint: disable-next=import-outside-toplevel
-        from homeassistant.components.zha.radio_manager import (
+        from homeassistant.components.zha import DOMAIN as ZHA_DOMAIN  # noqa: PLC0415
+        from homeassistant.components.zha.radio_manager import (  # noqa: PLC0415
             ZhaMultiPANMigrationHelper,
         )
-
-        # pylint: disable-next=import-outside-toplevel
-        from homeassistant.components.zha.silabs_multiprotocol import (
+        from homeassistant.components.zha.silabs_multiprotocol import (  # noqa: PLC0415
             async_get_channel as async_get_zha_channel,
         )
 
@@ -509,13 +512,13 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_start_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Start Silicon Labs Multiprotocol add-on."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
 
         if not self.start_task:
             self.start_task = self.hass.async_create_task(
-                multipan_manager.async_start_addon_waiting()
+                multipan_manager.async_start_addon_waiting(), eager_start=False
             )
 
         if not self.start_task.done():
@@ -538,7 +541,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_start_failed(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Add-on start failed."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
         return self.async_abort(
@@ -548,11 +551,12 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_finish_addon_setup(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Prepare info needed to complete the config entry update."""
         # Always reload entry after installing the addon.
         self.hass.async_create_task(
-            self.hass.config_entries.async_reload(self.config_entry.entry_id)
+            self.hass.config_entries.async_reload(self.config_entry.entry_id),
+            eager_start=False,
         )
 
         # Finish ZHA migration if needed
@@ -567,7 +571,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_addon_installed_other_device(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Show dialog explaining the addon is in use by another device."""
         if user_input is None:
             return self.async_show_form(step_id="addon_installed_other_device")
@@ -575,7 +579,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_addon_installed(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle logic when the addon is already installed."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
         addon_info = await self._async_get_addon_info(multipan_manager)
@@ -587,7 +591,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_addon_menu(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Show menu options for the addon."""
         return self.async_show_menu(
             step_id="addon_menu",
@@ -599,7 +603,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_reconfigure_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Reconfigure the addon."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
         active_platforms = await multipan_manager.async_active_platforms()
@@ -609,7 +613,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_notify_unknown_multipan_user(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Notify that there may be unknown multipan platforms."""
         if user_input is None:
             return self.async_show_form(
@@ -619,7 +623,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_change_channel(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Change the channel."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
         if user_input is None:
@@ -651,7 +655,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_notify_channel_change(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Notify that the channel change will take about five minutes."""
         if user_input is None:
             return self.async_show_form(
@@ -664,7 +668,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_uninstall_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Uninstall the addon and revert the firmware."""
         if user_input is None:
             return self.async_show_form(
@@ -681,7 +685,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_firmware_revert(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Install the flasher addon, if necessary."""
 
         flasher_manager = get_flasher_addon_manager(self.hass)
@@ -701,7 +705,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_install_flasher_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Show progress dialog for installing flasher addon."""
         flasher_manager = get_flasher_addon_manager(self.hass)
         addon_info = await self._async_get_addon_info(flasher_manager)
@@ -712,6 +716,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
             self.install_task = self.hass.async_create_task(
                 flasher_manager.async_install_addon_waiting(),
                 "SiLabs Flasher addon install",
+                eager_start=False,
             )
 
         if not self.install_task.done():
@@ -734,13 +739,10 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_configure_flasher_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Perform initial backup and reconfigure ZHA."""
-        # pylint: disable-next=import-outside-toplevel
-        from homeassistant.components.zha import DOMAIN as ZHA_DOMAIN
-
-        # pylint: disable-next=import-outside-toplevel
-        from homeassistant.components.zha.radio_manager import (
+        from homeassistant.components.zha import DOMAIN as ZHA_DOMAIN  # noqa: PLC0415
+        from homeassistant.components.zha.radio_manager import (  # noqa: PLC0415
             ZhaMultiPANMigrationHelper,
         )
 
@@ -794,7 +796,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_uninstall_multiprotocol_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Uninstall Silicon Labs Multiprotocol add-on."""
         multipan_manager = await get_multiprotocol_addon_manager(self.hass)
 
@@ -802,6 +804,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
             self.stop_task = self.hass.async_create_task(
                 multipan_manager.async_uninstall_addon_waiting(),
                 "SiLabs Multiprotocol addon uninstall",
+                eager_start=False,
             )
 
         if not self.stop_task.done():
@@ -821,7 +824,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_start_flasher_addon(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Start Silicon Labs Flasher add-on."""
         flasher_manager = get_flasher_addon_manager(self.hass)
 
@@ -834,7 +837,9 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
                     AddonState.NOT_RUNNING
                 )
 
-            self.start_task = self.hass.async_create_task(start_and_wait_until_done())
+            self.start_task = self.hass.async_create_task(
+                start_and_wait_until_done(), eager_start=False
+            )
 
         if not self.start_task.done():
             return self.async_show_progress(
@@ -856,7 +861,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_flasher_failed(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Flasher add-on start failed."""
         flasher_manager = get_flasher_addon_manager(self.hass)
         return self.async_abort(
@@ -866,7 +871,7 @@ class OptionsFlowHandler(config_entries.OptionsFlow, ABC):
 
     async def async_step_flashing_complete(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Finish flashing and update the config entry."""
         flasher_manager = get_flasher_addon_manager(self.hass)
         await flasher_manager.async_uninstall_addon_waiting()
