@@ -1,4 +1,5 @@
 """JustNimbus coordinator."""
+
 from __future__ import annotations
 
 from datetime import timedelta
@@ -15,20 +16,28 @@ from .const import CONF_ZIP_CODE, DOMAIN
 
 _LOGGER = logging.getLogger(__name__)
 
+type JustNimbusConfigEntry = ConfigEntry[JustNimbusCoordinator]
+
 
 class JustNimbusCoordinator(DataUpdateCoordinator[justnimbus.JustNimbusModel]):
     """Data update coordinator."""
 
-    def __init__(self, hass: HomeAssistant, entry: ConfigEntry) -> None:
+    config_entry: JustNimbusConfigEntry
+
+    def __init__(
+        self, hass: HomeAssistant, config_entry: JustNimbusConfigEntry
+    ) -> None:
         """Initialize the coordinator."""
         super().__init__(
             hass,
             _LOGGER,
+            config_entry=config_entry,
             name=DOMAIN,
             update_interval=timedelta(minutes=1),
         )
         self._client = justnimbus.JustNimbusClient(
-            client_id=entry.data[CONF_CLIENT_ID], zip_code=entry.data[CONF_ZIP_CODE]
+            client_id=config_entry.data[CONF_CLIENT_ID],
+            zip_code=config_entry.data[CONF_ZIP_CODE],
         )
 
     async def _async_update_data(self) -> justnimbus.JustNimbusModel:
