@@ -1,19 +1,16 @@
 """Config flow for LastFm."""
+
 from __future__ import annotations
 
+import logging
 from typing import Any
 
 from pylast import LastFMNetwork, PyLastError, User, WSError
 import voluptuous as vol
 
-from homeassistant.config_entries import (
-    ConfigEntry,
-    ConfigFlow,
-    OptionsFlowWithConfigEntry,
-)
+from homeassistant.config_entries import ConfigFlow, ConfigFlowResult, OptionsFlow
 from homeassistant.const import CONF_API_KEY
 from homeassistant.core import callback
-from homeassistant.data_entry_flow import FlowResult
 from homeassistant.helpers.selector import (
     SelectOptionDict,
     SelectSelector,
@@ -21,6 +18,7 @@ from homeassistant.helpers.selector import (
 )
 
 from .const import CONF_MAIN_USER, CONF_USERS, DOMAIN
+from .coordinator import LastFMConfigEntry
 
 PLACEHOLDERS = {"api_account_url": "https://www.last.fm/api/account/create"}
 
@@ -31,6 +29,8 @@ CONFIG_SCHEMA: vol.Schema = vol.Schema(
     }
 )
 
+_LOGGER = logging.getLogger(__name__)
+
 
 def get_lastfm_user(api_key: str, username: str) -> tuple[User, dict[str, str]]:
     """Get and validate lastFM User."""
@@ -48,7 +48,8 @@ def get_lastfm_user(api_key: str, username: str) -> tuple[User, dict[str, str]]:
             errors["base"] = "invalid_auth"
         else:
             errors["base"] = "unknown"
-    except Exception:  # pylint:disable=broad-except
+    except Exception:
+        _LOGGER.exception("Unexpected exception")
         errors["base"] = "unknown"
     return user, errors
 
@@ -76,14 +77,14 @@ class LastFmConfigFlowHandler(ConfigFlow, domain=DOMAIN):
     @staticmethod
     @callback
     def async_get_options_flow(
-        config_entry: ConfigEntry,
+        config_entry: LastFMConfigEntry,
     ) -> LastFmOptionsFlowHandler:
         """Get the options flow for this handler."""
-        return LastFmOptionsFlowHandler(config_entry)
+        return LastFmOptionsFlowHandler()
 
     async def async_step_user(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Initialize user input."""
         errors: dict[str, str] = {}
         if user_input is not None:
@@ -102,7 +103,7 @@ class LastFmConfigFlowHandler(ConfigFlow, domain=DOMAIN):
 
     async def async_step_friends(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Form to select other users and friends."""
         errors: dict[str, str] = {}
         if user_input is not None:
@@ -154,32 +155,35 @@ class LastFmConfigFlowHandler(ConfigFlow, domain=DOMAIN):
         )
 
 
-class LastFmOptionsFlowHandler(OptionsFlowWithConfigEntry):
+class LastFmOptionsFlowHandler(OptionsFlow):
     """LastFm Options flow handler."""
 
+    config_entry: LastFMConfigEntry
+
     async def async_step_init(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Initialize form."""
         errors: dict[str, str] = {}
+        options = self.config_entry.options
         if user_input is not None:
             users, errors = validate_lastfm_users(
-                self.options[CONF_API_KEY], user_input[CONF_USERS]
+                options[CONF_API_KEY], user_input[CONF_USERS]
             )
             user_input[CONF_USERS] = users
             if not errors:
                 return self.async_create_entry(
                     title="LastFM",
                     data={
-                        **self.options,
+                        **options,
                         CONF_USERS: user_input[CONF_USERS],
                     },
                 )
-        if self.options[CONF_MAIN_USER]:
+        if options[CONF_MAIN_USER]:
             try:
                 main_user, _ = get_lastfm_user(
-                    self.options[CONF_API_KEY],
-                    self.options[CONF_MAIN_USER],
+                    options[CONF_API_KEY],
+                    options[CONF_MAIN_USER],
                 )
                 friends_response = await self.hass.async_add_executor_job(
                     main_user.get_friends
@@ -205,6 +209,6 @@ class LastFmOptionsFlowHandler(OptionsFlowWithConfigEntry):
                         ),
                     }
                 ),
-                user_input or self.options,
+                user_input or options,
             ),
         )
