@@ -14,26 +14,36 @@ from homeassistant.components.sensor import (
     SensorEntityDescription,
     SensorStateClass,
 )
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import UnitOfTemperature
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
-from homeassistant.helpers.update_coordinator import (
-    CoordinatorEntity,
-    DataUpdateCoordinator,
-)
+from homeassistant.helpers.update_coordinator import CoordinatorEntity
 from homeassistant.util import dt as dt_util
 
-from .const import DOMAIN
+from . import MeaterCoordinator
+from .const import DOMAIN, MEATER_DATA
+from .coordinator import MeaterConfigEntry
+
+COOK_STATES = {
+    "Not Started": "not_started",
+    "Configured": "configured",
+    "Started": "started",
+    "Ready For Resting": "ready_for_resting",
+    "Resting": "resting",
+    "Slightly Underdone": "slightly_underdone",
+    "Finished": "finished",
+    "Slightly Overdone": "slightly_overdone",
+    "OVERCOOK!": "overcooked",
+}
 
 
 @dataclass(frozen=True, kw_only=True)
 class MeaterSensorEntityDescription(SensorEntityDescription):
     """Describes meater sensor entity."""
 
-    available: Callable[[MeaterProbe | None], bool]
     value: Callable[[MeaterProbe], datetime | float | str | None]
+    unavailable_when_not_cooking: bool = False
 
 
 def _elapsed_time_to_timestamp(probe: MeaterProbe) -> datetime | None:
@@ -62,7 +72,6 @@ SENSOR_TYPES = (
         device_class=SensorDeviceClass.TEMPERATURE,
         native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         state_class=SensorStateClass.MEASUREMENT,
-        available=lambda probe: probe is not None,
         value=lambda probe: probe.ambient_temperature,
     ),
     # Internal temperature (probe tip)
@@ -72,23 +81,22 @@ SENSOR_TYPES = (
         device_class=SensorDeviceClass.TEMPERATURE,
         native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         state_class=SensorStateClass.MEASUREMENT,
-        available=lambda probe: probe is not None,
         value=lambda probe: probe.internal_temperature,
     ),
     # Name of selected meat in user language or user given custom name
     MeaterSensorEntityDescription(
         key="cook_name",
         translation_key="cook_name",
-        available=lambda probe: probe is not None and probe.cook is not None,
+        unavailable_when_not_cooking=True,
         value=lambda probe: probe.cook.name if probe.cook else None,
     ),
-    # One of Not Started, Configured, Started, Ready For Resting, Resting,
-    # Slightly Underdone, Finished, Slightly Overdone, OVERCOOK!. Not translated.
     MeaterSensorEntityDescription(
         key="cook_state",
         translation_key="cook_state",
-        available=lambda probe: probe is not None and probe.cook is not None,
-        value=lambda probe: probe.cook.state if probe.cook else None,
+        unavailable_when_not_cooking=True,
+        device_class=SensorDeviceClass.ENUM,
+        options=list(COOK_STATES.values()),
+        value=lambda probe: COOK_STATES.get(probe.cook.state) if probe.cook else None,
     ),
     # Target temperature
     MeaterSensorEntityDescription(
@@ -97,10 +105,12 @@ SENSOR_TYPES = (
         device_class=SensorDeviceClass.TEMPERATURE,
         native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         state_class=SensorStateClass.MEASUREMENT,
-        available=lambda probe: probe is not None and probe.cook is not None,
-        value=lambda probe: probe.cook.target_temperature
-        if probe.cook and hasattr(probe.cook, "target_temperature")
-        else None,
+        unavailable_when_not_cooking=True,
+        value=(
+            lambda probe: probe.cook.target_temperature
+            if probe.cook and hasattr(probe.cook, "target_temperature")
+            else None
+        ),
     ),
     # Peak temperature
     MeaterSensorEntityDescription(
@@ -109,10 +119,12 @@ SENSOR_TYPES = (
         device_class=SensorDeviceClass.TEMPERATURE,
         native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         state_class=SensorStateClass.MEASUREMENT,
-        available=lambda probe: probe is not None and probe.cook is not None,
-        value=lambda probe: probe.cook.peak_temperature
-        if probe.cook and hasattr(probe.cook, "peak_temperature")
-        else None,
+        unavailable_when_not_cooking=True,
+        value=(
+            lambda probe: probe.cook.peak_temperature
+            if probe.cook and hasattr(probe.cook, "peak_temperature")
+            else None
+        ),
     ),
     # Remaining time in seconds. When unknown/calculating default is used. Default: -1
     # Exposed as a TIMESTAMP sensor where the timestamp is current time + remaining time.
@@ -120,7 +132,7 @@ SENSOR_TYPES = (
         key="cook_time_remaining",
         translation_key="cook_time_remaining",
         device_class=SensorDeviceClass.TIMESTAMP,
-        available=lambda probe: probe is not None and probe.cook is not None,
+        unavailable_when_not_cooking=True,
         value=_remaining_time_to_timestamp,
     ),
     # Time since the start of cook in seconds. Default: 0. Exposed as a TIMESTAMP sensor
@@ -129,7 +141,7 @@ SENSOR_TYPES = (
         key="cook_time_elapsed",
         translation_key="cook_time_elapsed",
         device_class=SensorDeviceClass.TIMESTAMP,
-        available=lambda probe: probe is not None and probe.cook is not None,
+        unavailable_when_not_cooking=True,
         value=_elapsed_time_to_timestamp,
     ),
 )
@@ -137,13 +149,11 @@ SENSOR_TYPES = (
 
 async def async_setup_entry(
     hass: HomeAssistant,
-    entry: ConfigEntry,
+    entry: MeaterConfigEntry,
     async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up the entry."""
-    coordinator: DataUpdateCoordinator[dict[str, MeaterProbe]] = hass.data[DOMAIN][
-        entry.entry_id
-    ]["coordinator"]
+    coordinator = entry.runtime_data
 
     @callback
     def async_update_data():
@@ -153,7 +163,7 @@ async def async_setup_entry(
 
         devices = coordinator.data
         entities = []
-        known_probes: set = hass.data[DOMAIN]["known_probes"]
+        known_probes = hass.data[MEATER_DATA]
 
         # Add entities for temperature probes which we've not yet seen
         for device_id in devices:
@@ -174,17 +184,20 @@ async def async_setup_entry(
 
     # Add a subscriber to the coordinator to discover new temperature probes
     coordinator.async_add_listener(async_update_data)
+    async_update_data()
 
 
-class MeaterProbeTemperature(
-    SensorEntity, CoordinatorEntity[DataUpdateCoordinator[dict[str, MeaterProbe]]]
-):
+class MeaterProbeTemperature(SensorEntity, CoordinatorEntity[MeaterCoordinator]):
     """Meater Temperature Sensor Entity."""
 
+    _attr_has_entity_name = True
     entity_description: MeaterSensorEntityDescription
 
     def __init__(
-        self, coordinator, device_id, description: MeaterSensorEntityDescription
+        self,
+        coordinator: MeaterCoordinator,
+        device_id: str,
+        description: MeaterSensorEntityDescription,
     ) -> None:
         """Initialise the sensor."""
         super().__init__(coordinator)
@@ -195,7 +208,7 @@ class MeaterProbeTemperature(
             },
             manufacturer="Apption Labs",
             model="Meater Probe",
-            name=f"Meater Probe {device_id}",
+            name=f"Meater Probe {device_id[:8]}",
         )
         self._attr_unique_id = f"{device_id}-{description.key}"
 
@@ -203,20 +216,24 @@ class MeaterProbeTemperature(
         self.entity_description = description
 
     @property
-    def native_value(self):
-        """Return the temperature of the probe."""
-        if not (device := self.coordinator.data.get(self.device_id)):
-            return None
+    def probe(self) -> MeaterProbe:
+        """Return the probe."""
+        return self.coordinator.data[self.device_id]
 
-        return self.entity_description.value(device)
+    @property
+    def native_value(self) -> datetime | float | str | None:
+        """Return the temperature of the probe."""
+        return self.entity_description.value(self.probe)
 
     @property
     def available(self) -> bool:
         """Return if entity is available."""
         # See if the device was returned from the API. If not, it's offline
         return (
-            self.coordinator.last_update_success
-            and self.entity_description.available(
-                self.coordinator.data.get(self.device_id)
+            super().available
+            and self.device_id in self.coordinator.data
+            and (
+                not self.entity_description.unavailable_when_not_cooking
+                or self.probe.cook is not None
             )
         )
