@@ -248,19 +248,22 @@ class NetatmoThermostat(NetatmoRoomEntity, ClimateEntity):
         if self.home.entity_id != data["home_id"]:
             return
 
-        if data["event_type"] == EVENT_TYPE_SCHEDULE and "schedule_id" in data:
-            self._selected_schedule = getattr(
-                self.hass.data[DOMAIN][DATA_SCHEDULES][self.home.entity_id].get(
-                    data["schedule_id"]
-                ),
-                "name",
-                None,
-            )
-            self._attr_extra_state_attributes[ATTR_SELECTED_SCHEDULE] = (
-                self._selected_schedule
-            )
-            self.async_write_ha_state()
-            self.data_handler.async_force_update(self._signal_name)
+        if data["event_type"] == EVENT_TYPE_SCHEDULE:
+            # handle schedule change
+            if "schedule_id" in data:
+                self._selected_schedule = getattr(
+                    self.hass.data[DOMAIN][DATA_SCHEDULES][self.home.entity_id].get(
+                        data["schedule_id"]
+                    ),
+                    "name",
+                    None,
+                )
+                self._attr_extra_state_attributes[ATTR_SELECTED_SCHEDULE] = (
+                    self._selected_schedule
+                )
+                self.async_write_ha_state()
+                self.data_handler.async_force_update(self._signal_name)
+            # ignore other schedule events
             return
 
         home = data["home"]
