@@ -27,8 +27,14 @@ from homeassistant.helpers.aiohttp_client import async_get_clientsession
 if TYPE_CHECKING:
     from . import RenaultConfigEntry
 
-from .const import CONF_KAMEREON_ACCOUNT_ID, DEFAULT_SCAN_INTERVAL
-from .renault_vehicle import RenaultVehicleProxy
+from time import time
+
+from .const import (
+    CONF_KAMEREON_ACCOUNT_ID,
+    COOLING_UPDATES_SECONDS,
+    MAX_CALLS_PER_HOURS,
+)
+from .renault_vehicle import COORDINATORS, RenaultVehicleProxy
 
 LOGGER = logging.getLogger(__name__)
 
@@ -45,6 +51,24 @@ class RenaultHub:
         self._account: RenaultAccount | None = None
         self._vehicles: dict[str, RenaultVehicleProxy] = {}
 
+        self._got_throttled_at_time: float | None = None
+
+    def set_throttled(self) -> None:
+        """We got throttled, we need to adjust the rate limit."""
+        if self._got_throttled_at_time is None:
+            self._got_throttled_at_time = time()
+
+    def is_throttled(self) -> bool:
+        """Check if we are throttled."""
+        if self._got_throttled_at_time is None:
+            return False
+
+        if time() - self._got_throttled_at_time > COOLING_UPDATES_SECONDS:
+            self._got_throttled_at_time = None
+            return False
+
+        return True
+
     async def attempt_login(self, username: str, password: str) -> bool:
         """Attempt login to Renault servers."""
         try:
@@ -58,7 +82,6 @@ class RenaultHub:
     async def async_initialise(self, config_entry: RenaultConfigEntry) -> None:
         """Set up proxy."""
         account_id: str = config_entry.data[CONF_KAMEREON_ACCOUNT_ID]
-        scan_interval = timedelta(seconds=DEFAULT_SCAN_INTERVAL)
 
         self._account = await self._client.get_api_account(account_id)
         vehicles = await self._account.get_vehicles()
@@ -70,6 +93,12 @@ class RenaultHub:
                 raise ConfigEntryNotReady(
                     "Failed to retrieve vehicle details from Renault servers"
                 )
+
+            num_call_per_scan = len(COORDINATORS) * len(vehicles.vehicleLinks)
+            scan_interval = timedelta(
+                seconds=(3600 * num_call_per_scan) / MAX_CALLS_PER_HOURS
+            )
+
             device_registry = dr.async_get(self._hass)
             await asyncio.gather(
                 *(
@@ -84,6 +113,21 @@ class RenaultHub:
                 )
             )
 
+            # all vehicles have been initiated with the right number of active coordinators
+            num_call_per_scan = 0
+            for vehicle_link in vehicles.vehicleLinks:
+                vehicle = self._vehicles[str(vehicle_link.vin)]
+                num_call_per_scan += len(vehicle.coordinators)
+
+            new_scan_interval = timedelta(
+                seconds=(3600 * num_call_per_scan) / MAX_CALLS_PER_HOURS
+            )
+            if new_scan_interval != scan_interval:
+                # we need to change the vehicles with the right scan interval
+                for vehicle_link in vehicles.vehicleLinks:
+                    vehicle = self._vehicles[str(vehicle_link.vin)]
+                    vehicle.update_scan_interval(new_scan_interval)
+
     async def async_initialise_vehicle(
         self,
         vehicle_link: KamereonVehiclesLink,
@@ -99,6 +143,7 @@ class RenaultHub:
         vehicle = RenaultVehicleProxy(
             hass=self._hass,
             config_entry=config_entry,
+            hub=self,
             vehicle=await renault_account.get_api_vehicle(vehicle_link.vin),
             details=vehicle_link.vehicleDetails,
             scan_interval=scan_interval,
