@@ -1,7 +1,8 @@
 """Support for SimpliSafe locks."""
+
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any
+from typing import Any
 
 from simplipy.device.lock import Lock, LockStates
 from simplipy.errors import SimplipyError
@@ -11,10 +12,12 @@ from simplipy.websocket import EVENT_LOCK_LOCKED, EVENT_LOCK_UNLOCKED, Websocket
 from homeassistant.components.lock import LockEntity
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.exceptions import HomeAssistantError
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
-from . import SimpliSafe, SimpliSafeEntity
+from . import SimpliSafe
 from .const import DOMAIN, LOGGER
+from .entity import SimpliSafeEntity
 
 ATTR_LOCK_LOW_BATTERY = "lock_low_battery"
 ATTR_PIN_PAD_LOW_BATTERY = "pin_pad_low_battery"
@@ -28,19 +31,22 @@ WEBSOCKET_EVENTS_TO_LISTEN_FOR = (EVENT_LOCK_LOCKED, EVENT_LOCK_UNLOCKED)
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: ConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up SimpliSafe locks based on a config entry."""
     simplisafe = hass.data[DOMAIN][entry.entry_id]
-    locks = []
+    locks: list[SimpliSafeLock] = []
 
     for system in simplisafe.systems.values():
         if system.version == 2:
-            LOGGER.info("Skipping lock setup for V2 system: %s", system.system_id)
+            LOGGER.warning("Skipping lock setup for V2 system: %s", system.system_id)
             continue
 
-        for lock in system.locks.values():
-            locks.append(SimpliSafeLock(simplisafe, system, lock))
+        locks.extend(
+            SimpliSafeLock(simplisafe, system, lock) for lock in system.locks.values()
+        )
 
     async_add_entities(locks)
 
@@ -48,6 +54,9 @@ async def async_setup_entry(
 class SimpliSafeLock(SimpliSafeEntity, LockEntity):
     """Define a SimpliSafe lock."""
 
+    _attr_name = None
+    _device: Lock
+
     def __init__(self, simplisafe: SimpliSafe, system: SystemV3, lock: Lock) -> None:
         """Initialize."""
         super().__init__(
@@ -57,15 +66,14 @@ class SimpliSafeLock(SimpliSafeEntity, LockEntity):
             additional_websocket_events=WEBSOCKET_EVENTS_TO_LISTEN_FOR,
         )
 
-        self._device: Lock
-
     async def async_lock(self, **kwargs: Any) -> None:
         """Lock the lock."""
         try:
             await self._device.async_lock()
         except SimplipyError as err:
-            LOGGER.error('Error while locking "%s": %s', self._device.name, err)
-            return
+            raise HomeAssistantError(
+                f'Error while locking "{self._device.name}": {err}'
+            ) from err
 
         self._attr_is_locked = True
         self.async_write_ha_state()
@@ -75,8 +83,9 @@ class SimpliSafeLock(SimpliSafeEntity, LockEntity):
         try:
             await self._device.async_unlock()
         except SimplipyError as err:
-            LOGGER.error('Error while unlocking "%s": %s', self._device.name, err)
-            return
+            raise HomeAssistantError(
+                f'Error while unlocking "{self._device.name}": {err}'
+            ) from err
 
         self._attr_is_locked = False
         self.async_write_ha_state()
@@ -97,8 +106,8 @@ class SimpliSafeLock(SimpliSafeEntity, LockEntity):
     @callback
     def async_update_from_websocket_event(self, event: WebsocketEvent) -> None:
         """Update the entity when new data comes from the websocket."""
-        if TYPE_CHECKING:
-            assert event.event_type
+        assert event.event_type
+
         if state := STATE_MAP_FROM_WEBSOCKET_EVENT.get(event.event_type) is not None:
             self._attr_is_locked = state
             self.async_reset_error_count()
