@@ -1,4 +1,5 @@
 """Template platform that aggregates meteorological data."""
+
 from __future__ import annotations
 
 from dataclasses import asdict, dataclass
@@ -25,12 +26,12 @@ from homeassistant.components.weather import (
     ATTR_CONDITION_WINDY_VARIANT,
     DOMAIN as WEATHER_DOMAIN,
     ENTITY_ID_FORMAT,
+    PLATFORM_SCHEMA as WEATHER_PLATFORM_SCHEMA,
     Forecast,
     WeatherEntity,
     WeatherEntityFeature,
 )
 from homeassistant.const import (
-    CONF_NAME,
     CONF_TEMPERATURE_UNIT,
     CONF_UNIQUE_ID,
     STATE_UNAVAILABLE,
@@ -38,9 +39,7 @@ from homeassistant.const import (
 )
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.exceptions import TemplateError
-from homeassistant.helpers import template
-import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.config_validation import PLATFORM_SCHEMA
+from homeassistant.helpers import config_validation as cv, template
 from homeassistant.helpers.entity import async_generate_entity_id
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.restore_state import ExtraStoredData, RestoreEntity
@@ -53,7 +52,11 @@ from homeassistant.util.unit_conversion import (
 )
 
 from .coordinator import TriggerUpdateCoordinator
-from .template_entity import TemplateEntity, rewrite_common_legacy_to_modern_conf
+from .template_entity import (
+    TemplateEntity,
+    make_template_entity_common_modern_schema,
+    rewrite_common_legacy_to_modern_conf,
+)
 from .trigger_entity import TriggerEntity
 
 CHECK_FORECAST_KEYS = (
@@ -92,7 +95,6 @@ CONF_WIND_SPEED_TEMPLATE = "wind_speed_template"
 CONF_WIND_BEARING_TEMPLATE = "wind_bearing_template"
 CONF_OZONE_TEMPLATE = "ozone_template"
 CONF_VISIBILITY_TEMPLATE = "visibility_template"
-CONF_FORECAST_TEMPLATE = "forecast_template"
 CONF_FORECAST_DAILY_TEMPLATE = "forecast_daily_template"
 CONF_FORECAST_HOURLY_TEMPLATE = "forecast_hourly_template"
 CONF_FORECAST_TWICE_DAILY_TEMPLATE = "forecast_twice_daily_template"
@@ -105,39 +107,62 @@ CONF_CLOUD_COVERAGE_TEMPLATE = "cloud_coverage_template"
 CONF_DEW_POINT_TEMPLATE = "dew_point_template"
 CONF_APPARENT_TEMPERATURE_TEMPLATE = "apparent_temperature_template"
 
+DEFAULT_NAME = "Template Weather"
+
 WEATHER_SCHEMA = vol.Schema(
     {
-        vol.Required(CONF_NAME): cv.template,
-        vol.Required(CONF_CONDITION_TEMPLATE): cv.template,
-        vol.Required(CONF_TEMPERATURE_TEMPLATE): cv.template,
-        vol.Required(CONF_HUMIDITY_TEMPLATE): cv.template,
+        vol.Optional(CONF_APPARENT_TEMPERATURE_TEMPLATE): cv.template,
         vol.Optional(CONF_ATTRIBUTION_TEMPLATE): cv.template,
-        vol.Optional(CONF_PRESSURE_TEMPLATE): cv.template,
-        vol.Optional(CONF_WIND_SPEED_TEMPLATE): cv.template,
-        vol.Optional(CONF_WIND_BEARING_TEMPLATE): cv.template,
-        vol.Optional(CONF_OZONE_TEMPLATE): cv.template,
-        vol.Optional(CONF_VISIBILITY_TEMPLATE): cv.template,
-        vol.Optional(CONF_FORECAST_TEMPLATE): cv.template,
+        vol.Optional(CONF_CLOUD_COVERAGE_TEMPLATE): cv.template,
+        vol.Required(CONF_CONDITION_TEMPLATE): cv.template,
+        vol.Optional(CONF_DEW_POINT_TEMPLATE): cv.template,
+        vol.Required(CONF_HUMIDITY_TEMPLATE): cv.template,
         vol.Optional(CONF_FORECAST_DAILY_TEMPLATE): cv.template,
         vol.Optional(CONF_FORECAST_HOURLY_TEMPLATE): cv.template,
         vol.Optional(CONF_FORECAST_TWICE_DAILY_TEMPLATE): cv.template,
-        vol.Optional(CONF_UNIQUE_ID): cv.string,
-        vol.Optional(CONF_TEMPERATURE_UNIT): vol.In(TemperatureConverter.VALID_UNITS),
-        vol.Optional(CONF_PRESSURE_UNIT): vol.In(PressureConverter.VALID_UNITS),
-        vol.Optional(CONF_WIND_SPEED_UNIT): vol.In(SpeedConverter.VALID_UNITS),
-        vol.Optional(CONF_VISIBILITY_UNIT): vol.In(DistanceConverter.VALID_UNITS),
+        vol.Optional(CONF_OZONE_TEMPLATE): cv.template,
         vol.Optional(CONF_PRECIPITATION_UNIT): vol.In(DistanceConverter.VALID_UNITS),
+        vol.Optional(CONF_PRESSURE_TEMPLATE): cv.template,
+        vol.Optional(CONF_PRESSURE_UNIT): vol.In(PressureConverter.VALID_UNITS),
+        vol.Required(CONF_TEMPERATURE_TEMPLATE): cv.template,
+        vol.Optional(CONF_TEMPERATURE_UNIT): vol.In(TemperatureConverter.VALID_UNITS),
+        vol.Optional(CONF_VISIBILITY_TEMPLATE): cv.template,
+        vol.Optional(CONF_VISIBILITY_UNIT): vol.In(DistanceConverter.VALID_UNITS),
+        vol.Optional(CONF_WIND_BEARING_TEMPLATE): cv.template,
         vol.Optional(CONF_WIND_GUST_SPEED_TEMPLATE): cv.template,
-        vol.Optional(CONF_CLOUD_COVERAGE_TEMPLATE): cv.template,
-        vol.Optional(CONF_DEW_POINT_TEMPLATE): cv.template,
-        vol.Optional(CONF_APPARENT_TEMPERATURE_TEMPLATE): cv.template,
+        vol.Optional(CONF_WIND_SPEED_TEMPLATE): cv.template,
+        vol.Optional(CONF_WIND_SPEED_UNIT): vol.In(SpeedConverter.VALID_UNITS),
     }
-)
+).extend(make_template_entity_common_modern_schema(DEFAULT_NAME).schema)
 
-PLATFORM_SCHEMA = vol.All(
-    cv.deprecated(CONF_FORECAST_TEMPLATE),
-    PLATFORM_SCHEMA.extend(WEATHER_SCHEMA.schema),
-)
+PLATFORM_SCHEMA = WEATHER_PLATFORM_SCHEMA.extend(WEATHER_SCHEMA.schema)
+
+
+@callback
+def _async_create_template_tracking_entities(
+    async_add_entities: AddEntitiesCallback,
+    hass: HomeAssistant,
+    definitions: list[dict],
+    unique_id_prefix: str | None,
+) -> None:
+    """Create the weather entities."""
+    entities = []
+
+    for entity_conf in definitions:
+        unique_id = entity_conf.get(CONF_UNIQUE_ID)
+
+        if unique_id and unique_id_prefix:
+            unique_id = f"{unique_id_prefix}-{unique_id}"
+
+        entities.append(
+            WeatherTemplate(
+                hass,
+                entity_conf,
+                unique_id,
+            )
+        )
+
+    async_add_entities(entities)
 
 
 async def async_setup_platform(
@@ -147,24 +172,32 @@ async def async_setup_platform(
     discovery_info: DiscoveryInfoType | None = None,
 ) -> None:
     """Set up the Template weather."""
-    if discovery_info and "coordinator" in discovery_info:
+    if discovery_info is None:
+        config = rewrite_common_legacy_to_modern_conf(hass, config)
+        unique_id = config.get(CONF_UNIQUE_ID)
+        async_add_entities(
+            [
+                WeatherTemplate(
+                    hass,
+                    config,
+                    unique_id,
+                )
+            ]
+        )
+        return
+
+    if "coordinator" in discovery_info:
         async_add_entities(
             TriggerWeatherEntity(hass, discovery_info["coordinator"], config)
             for config in discovery_info["entities"]
         )
         return
 
-    config = rewrite_common_legacy_to_modern_conf(config)
-    unique_id = config.get(CONF_UNIQUE_ID)
-
-    async_add_entities(
-        [
-            WeatherTemplate(
-                hass,
-                config,
-                unique_id,
-            )
-        ]
+    _async_create_template_tracking_entities(
+        async_add_entities,
+        hass,
+        discovery_info["entities"],
+        discovery_info["unique_id"],
     )
 
 
@@ -192,7 +225,6 @@ class WeatherTemplate(TemplateEntity, WeatherEntity):
         self._wind_bearing_template = config.get(CONF_WIND_BEARING_TEMPLATE)
         self._ozone_template = config.get(CONF_OZONE_TEMPLATE)
         self._visibility_template = config.get(CONF_VISIBILITY_TEMPLATE)
-        self._forecast_template = config.get(CONF_FORECAST_TEMPLATE)
         self._forecast_daily_template = config.get(CONF_FORECAST_DAILY_TEMPLATE)
         self._forecast_hourly_template = config.get(CONF_FORECAST_HOURLY_TEMPLATE)
         self._forecast_twice_daily_template = config.get(
@@ -226,7 +258,6 @@ class WeatherTemplate(TemplateEntity, WeatherEntity):
         self._cloud_coverage = None
         self._dew_point = None
         self._apparent_temperature = None
-        self._forecast: list[Forecast] = []
         self._forecast_daily: list[Forecast] = []
         self._forecast_hourly: list[Forecast] = []
         self._forecast_twice_daily: list[Forecast] = []
@@ -299,11 +330,6 @@ class WeatherTemplate(TemplateEntity, WeatherEntity):
         """Return the apparent temperature."""
         return self._apparent_temperature
 
-    @property
-    def forecast(self) -> list[Forecast]:
-        """Return the forecast."""
-        return self._forecast
-
     async def async_forecast_daily(self) -> list[Forecast]:
         """Return the daily forecast in native units."""
         return self._forecast_daily
@@ -393,11 +419,6 @@ class WeatherTemplate(TemplateEntity, WeatherEntity):
                 "_apparent_temperature",
                 self._apparent_temperature_template,
             )
-        if self._forecast_template:
-            self.add_template_attribute(
-                "_forecast",
-                self._forecast_template,
-            )
 
         if self._forecast_daily_template:
             self.add_template_attribute(
@@ -432,7 +453,9 @@ class WeatherTemplate(TemplateEntity, WeatherEntity):
         """Save template result and trigger forecast listener."""
         attr_result = None if isinstance(result, TemplateError) else result
         setattr(self, f"_forecast_{forecast_type}", attr_result)
-        self.hass.create_task(self.async_update_listeners([forecast_type]))
+        self.hass.async_create_task(
+            self.async_update_listeners([forecast_type]), eager_start=True
+        )
 
     @callback
     def _validate_forecast(
@@ -571,12 +594,12 @@ class TriggerWeatherEntity(TriggerEntity, WeatherEntity, RestoreEntity):
             and state.state not in (STATE_UNKNOWN, STATE_UNAVAILABLE)
             and (weather_data := await self.async_get_last_weather_data())
         ):
-            self._rendered[
-                CONF_APPARENT_TEMPERATURE_TEMPLATE
-            ] = weather_data.last_apparent_temperature
-            self._rendered[
-                CONF_CLOUD_COVERAGE_TEMPLATE
-            ] = weather_data.last_cloud_coverage
+            self._rendered[CONF_APPARENT_TEMPERATURE_TEMPLATE] = (
+                weather_data.last_apparent_temperature
+            )
+            self._rendered[CONF_CLOUD_COVERAGE_TEMPLATE] = (
+                weather_data.last_cloud_coverage
+            )
             self._rendered[CONF_CONDITION_TEMPLATE] = state.state
             self._rendered[CONF_DEW_POINT_TEMPLATE] = weather_data.last_dew_point
             self._rendered[CONF_HUMIDITY_TEMPLATE] = weather_data.last_humidity
@@ -585,9 +608,9 @@ class TriggerWeatherEntity(TriggerEntity, WeatherEntity, RestoreEntity):
             self._rendered[CONF_TEMPERATURE_TEMPLATE] = weather_data.last_temperature
             self._rendered[CONF_VISIBILITY_TEMPLATE] = weather_data.last_visibility
             self._rendered[CONF_WIND_BEARING_TEMPLATE] = weather_data.last_wind_bearing
-            self._rendered[
-                CONF_WIND_GUST_SPEED_TEMPLATE
-            ] = weather_data.last_wind_gust_speed
+            self._rendered[CONF_WIND_GUST_SPEED_TEMPLATE] = (
+                weather_data.last_wind_gust_speed
+            )
             self._rendered[CONF_WIND_SPEED_TEMPLATE] = weather_data.last_wind_speed
 
     @property
