@@ -1,4 +1,5 @@
 """Support for Verisure binary sensors."""
+
 from __future__ import annotations
 
 from homeassistant.components.binary_sensor import (
@@ -10,7 +11,7 @@ from homeassistant.const import ATTR_LAST_TRIP_TIME, EntityCategory
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity import Entity
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 from homeassistant.util import dt as dt_util
 
@@ -21,7 +22,7 @@ from .coordinator import VerisureDataUpdateCoordinator
 async def async_setup_entry(
     hass: HomeAssistant,
     entry: ConfigEntry,
-    async_add_entities: AddEntitiesCallback,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up Verisure binary sensors based on a config entry."""
     coordinator: VerisureDataUpdateCoordinator = hass.data[DOMAIN][entry.entry_id]
@@ -61,7 +62,7 @@ class VerisureDoorWindowSensor(
             manufacturer="Verisure",
             model="Shock Sensor Detector",
             identifiers={(DOMAIN, self.serial_number)},
-            via_device=(DOMAIN, self.coordinator.entry.data[CONF_GIID]),
+            via_device=(DOMAIN, self.coordinator.config_entry.data[CONF_GIID]),
             configuration_url="https://mypages.verisure.com",
         )
 
@@ -103,7 +104,7 @@ class VerisureEthernetStatus(
     @property
     def unique_id(self) -> str:
         """Return the unique ID for this entity."""
-        return f"{self.coordinator.entry.data[CONF_GIID]}_ethernet"
+        return f"{self.coordinator.config_entry.data[CONF_GIID]}_ethernet"
 
     @property
     def device_info(self) -> DeviceInfo:
@@ -112,7 +113,7 @@ class VerisureEthernetStatus(
             name="Verisure Alarm",
             manufacturer="Verisure",
             model="VBox",
-            identifiers={(DOMAIN, self.coordinator.entry.data[CONF_GIID])},
+            identifiers={(DOMAIN, self.coordinator.config_entry.data[CONF_GIID])},
             configuration_url="https://mypages.verisure.com",
         )
 
