@@ -1,4 +1,5 @@
 """Utilities to help with aiohttp."""
+
 from __future__ import annotations
 
 from http import HTTPStatus
@@ -27,6 +28,33 @@ class MockStreamReader:
         return self._content.read(byte_count)
 
 
+class MockStreamReaderChunked(MockStreamReader):
+    """Mock a stream reader with simulated chunked data."""
+
+    async def readchunk(self) -> tuple[bytes, bool]:
+        """Read bytes."""
+        return (self._content.read(), False)
+
+
+class MockPayloadWriter:
+    """Small mock to imitate payload writer."""
+
+    def enable_chunking(self) -> None:
+        """Enable chunking."""
+
+    def send_headers(self, *args: Any, **kwargs: Any) -> None:
+        """Write headers."""
+
+    async def write_headers(self, *args: Any, **kwargs: Any) -> None:
+        """Write headers."""
+
+    async def write(self, *args: Any, **kwargs: Any) -> None:
+        """Write payload."""
+
+
+_MOCK_PAYLOAD_WRITER = MockPayloadWriter()
+
+
 class MockRequest:
     """Mock an aiohttp request."""
 
@@ -48,8 +76,14 @@ class MockRequest:
         self.status = status
         self.headers: CIMultiDict[str] = CIMultiDict(headers or {})
         self.query_string = query_string or ""
+        self.keep_alive = False
+        self.version = (1, 1)
         self._content = content
         self.mock_source = mock_source
+        self._payload_writer = _MOCK_PAYLOAD_WRITER
+
+    async def _prepare_hook(self, response: Any) -> None:
+        """Prepare hook."""
 
     @property
     def query(self) -> MultiDict[str]:
@@ -89,8 +123,7 @@ def serialize_response(response: web.Response) -> dict[str, Any]:
     if (body := response.body) is None:
         body_decoded = None
     elif isinstance(body, payload.StringPayload):
-        # pylint: disable-next=protected-access
-        body_decoded = body._value.decode(body.encoding)
+        body_decoded = body._value.decode(body.encoding or "utf-8")  # noqa: SLF001
     elif isinstance(body, bytes):
         body_decoded = body.decode(response.charset or "utf-8")
     else:
